%
%   This function transforms the interpolation data for the
%    multiple Nevanlinna-Pick interpolation problem subjected
%    to the bilinear transformation
%
%            az + b                -du + b
%       u = --------   <==>   z = --------- 
%            cz + d                 cu - a
%
%   Input:  domaintrans  -  Transformation parameters
%           interdata    -  Interpolation data
%
%   Output: interdatanew  -  New interpolation data
%
function interdatanew = BilinearDomainData(domaintrans,interdata)

z = interdata.z;
W = interdata.W;
a = domaintrans.a;
b = domaintrans.b;
c = domaintrans.c;
d = domaintrans.d;

for k = 1:interdata.ntilde

    % Determine interpolation point and function value
    if abs(z(k)) > 1e10
        u(k) = a / c;
    else
        u(k) = (a*z(k) + b) / (c*z(k) + d);
    end
    V{k}{1} = W{k}{1};

    % Determine multiplicity
    mult = size(W{k},2);
    m = size(W{k}{1},1);

    if(mult >= 2)
        % Determine first derivative interpolation data
        zprim = (a*d - b*c) / (c*u(k) - a)^2;
        V{k}{2} = W{k}{2} * zprim;
    
        % Determine higher order derivative interpolation data (if any...)
        coeff{1} = 1;

        fact = 1;
        for l = 3:mult
            
            coeff{l-1} = 1;
            for j = 2:l-2
                coeff{l-1}(j) = (2*l-j-2)/j*coeff{l-2}(j-1) + coeff{l-2}(j); 
            end
            coeff{l-1}(end+1) = 1;
            
            zprim = [ zprim -zprim(end)*(l-1)/(c*u(k) - a)*c ];
            V{k}{l} = zeros(m,m);
            for j = 1:l-1
                V{k}{l} = V{k}{l} + coeff{l-1}(j)*W{k}{l-j+1}*factorial(l-j)*(zprim(1))^(l-j-1)*zprim(j);
            end
            V{k}{l} = V{k}{l}/factorial(l-1);
       end
    end
end

speczeros = [];
for s = interdata.speczeros
    if abs(s) > 1e10
        speczeros(end+1) = domaintrans.a / domaintrans.c;
    else
        speczeros(end+1) = (domaintrans.a * s + domaintrans.b) / ...
            (domaintrans.c * s + domaintrans.d);
    end
end

interdatanew = GetInterData(u,V,speczeros,interdata.maxdistancesp);