%
%   This function calculates the hessian of g_rho (or g~_rho)
%    at the point (R, mu)
%
%       g_rho(RR) = trace(RR*K*RR') - 2 < log det R, p >
%
%       where   R = R0 + R1 z + ... Rn z^n
%              RR = [ R0 R1 ... Rn ]
%                     mu
%               p = -------
%                   tau tau*
%              mu = mu0 + mu1 z + ... + mun z^n
%             tau = tau0 + tau1 z + ... + taun z^n
%
%   Input: K = K
%          R(:,:,k) = Rk
%          tau = [ tau0 ...taun ]
%          muvec = [ mu0 ... mun ]
%   
%   Output: H = d^2/(d Vec RR')^2 g_rho
%
function H = GetHessian(R,muvec,tau,K)

% Some sizes
m = size(R,1);
n = size(R,3)-1;
ntau = length(tau);

% T(z) = Rtau(z) (*) RT(z)
Rtau = MPolyMult(reshape(tau,1,1,ntau),R);
RT = BlkTrans(R);
B = MPolyMult(Rtau,RT);
B = BlkTrans(B);
T = GetARCovariances(B,3*n);

% U(z) = R* (*) R*'
U = MPolyMult(RT,R);

% V(z) = mu(z) * T(z)
muvec(1) = muvec(1) / 2;
V = kron(muvec',eye(m*m)) ...
    * ( BlkHankel(BlkTrans(T(:,:,1:n+1)),BlkTrans(T(:,:,n+1:end))) ... 
    + BlkToeplitz(T(:,:,1:n+1),BlkTrans(T(:,:,1:2*n+1))) );

% S(z) = U(z) * V(z)
S = reshape(U,m*m,m*m*(2*n+1)) * BlkToeplitz(reshape(V,m*m,m*m,2*n+1));

% Stilde
Stilde = ReOrder1(reshape(S,m*m,m*m,2*n+1));
Stilde = BlkHankel(Stilde(:,:,1:n+1),Stilde(:,:,n+1:end));
Stilde = ReOrder2(Stilde,n+1,m);

% Calculate hessian
H = 2*(kron(eye(m),K) + Stilde);

% Modify H to be more pos def
H = H + norm(H)*1e-6*eye(size(H,1));