%
%   This function returns an interpolation data set of the form
%    used in the npdcsolver.
%
%   Input: z - interpolation points
%          W - interpolation data
%          speczeros -  spectral zeros
%          maxdistancesp - the maximal distance from origin 
%                           for the spectral zeros
%
%   Output: interdata - an instance of the interpolation data set
%
%    interdata.z      = [ z0 z1 ... zn ]
%    interdata.W      = { W0 W1 ... Wn }
%                        Wi = { Wi0 Wi1 ... Wi(mi-1) }
%                        Wij = m-by-m matrix
%    interdata.Wvec   = { W0 W1 ... Wn }
%                        Wi = [ Wi0 Wi1 ... Wi(mi-1) ]
%    interdata.n      = number of interpolation points minus one
%    interdata.ntilde = number of interpolation points
%    interdata.mw     = multiplicities of each interpolation point
%    interdata.mkn    = number of interpolation points taken
%                        with multiplicity
%    interdata.m      = dimension of Wij
%    interdata.tau    = polynomial increasing order
%                                 __      __
%                       tau(z) =  || (1 - zk z)^mk
%                               k=0..m
%    interdata.speczeros = spectral zeros
%    interdata.maxdistancesp = the maximal distance from origin 
%                               for the spectral zeros (once in npdc)
%
function interdata = GetInterData(z,W,speczeros,maxdistancesp)

% Check that the sizes are ok
if size(z,1) ~= 1 | size(W,1) ~= 1
    disp('In getinterdata: z & w must be 1 times something')
    return
elseif size(z,2) ~= size(W,2)
    disp('In getinterdata: z & w must have the same length')
    return
end

% Check that the z:s are distinct
for k = 1:size(z,2)
    for l = k+1:size(z,2)
        if abs(z(k) - z(l)) < 10*eps
            disp('In getinterdata: the z:s must be distinct')
            return
        end
    end
end

% Check that the Wk:s are nonempty and construct the Wvec
for k = 1: size(W,2)
    if isempty(W{k})
        disp('In getinterdata: the Wk:s must be non-empty')
        return
    end
    mw(k) = size(W{k},2);
    
    Wvec{k} = [];
    for l = 1:mw(k)
        Wvec{k} = [ Wvec{k} W{k}{l} ];
    end
end

% Reorder if there is a z = 0
index = find(z==0);
if index
    indexcomp = find(z~=0);
    z = z([index indexcomp]);
    W = W([index indexcomp]);
    Wvec = Wvec([index indexcomp]);
    mw = mw([index indexcomp]);
end

%   Construct tau
p = [];
noatend = 0;
for k = 1:length(z)
    if norm(z(k)) < 10*eps
        noatend = noatend + mw(k) - 1;
    elseif norm(z(k)) > 1e10
        noatend = noatend + mw(k);
    else
        p = [ p z(k)*ones(1,mw(k)) ];
    end
end
tau = poly(p);  % Note poly([]) = 1;
tau = real( fliplr(tau) / tau(end) );
tau = [ tau zeros(1,noatend) ];

%   Construct tau
p = [];
noatend = 0;
for k = 1:length(z)
    if norm(z(k)) < 10*eps
        noatend = noatend + mw(k) - 1;
    elseif norm(z(k)) > 1e10
        noatend = noatend + mw(k);
    else
        p = [ p 1/conj(z(k))*ones(1,mw(k)) ];
    end
end
tau = poly(p);  % Note poly([]) = 1;
tau = real( fliplr(tau) / tau(end) );
tau = [ tau zeros(1,noatend) ];


interdata.z = z;
interdata.W = W;
interdata.Wvec = Wvec;
interdata.ntilde = size(z,2);
interdata.n = interdata.ntilde - 1;
interdata.mkn = sum(mw);
interdata.tau = tau;
interdata.mw = mw;
interdata.m = size(W{1}{1},1);
interdata.speczeros = speczeros;
interdata.maxdistancesp = maxdistancesp;